/*
 * LCD780
 *   LCD-
 *
 *
 *   .
 *
 * http://mypractic.ru
 */

#include "LCD780.h"
#include "DelayDWT.h"


//  
void noDisplay_LCD() {
	_GPIOx_RS -> BSRR = _GPIO_Pin_RS << 16; //   RS
	_displaycontrol &= 0b11111011;
	writeByte_LCD(_displaycontrol);
	delayMks(50);
}

//  
void display_LCD() {
	_GPIOx_RS -> BSRR = _GPIO_Pin_RS << 16; //   RS
	_displaycontrol |= 0b00001100;
	writeByte_LCD(_displaycontrol);
	delayMks(50);
}

//  
void noCursor_LCD() {
	_GPIOx_RS -> BSRR = _GPIO_Pin_RS << 16; //   RS
	_displaycontrol &= 0b11111101;
	writeByte_LCD(_displaycontrol);
	delayMks(50);
}

//  
void cursor_LCD() {
	_GPIOx_RS -> BSRR = _GPIO_Pin_RS << 16; //   RS
	_displaycontrol |= 0b00001010;
	writeByte_LCD(_displaycontrol);
	delayMks(50);
}

//   
void noBlink_LCD() {
	_GPIOx_RS -> BSRR = _GPIO_Pin_RS << 16; //   RS
	_displaycontrol &= 0b11111110;
	writeByte_LCD(_displaycontrol);
	delayMks(50);
}

//   
void blink_LCD() {
	_GPIOx_RS -> BSRR = _GPIO_Pin_RS << 16; //   RS
	_displaycontrol |= 0b00001001;
	writeByte_LCD(_displaycontrol);
	delayMks(50);
}

//  
void scrollDisplayLeft_LCD() {
	_GPIOx_RS -> BSRR = _GPIO_Pin_RS << 16; //   RS
	writeByte_LCD(0x18);
	delayMks(50);
}

//  
void scrollDisplayRight_LCD() {
	_GPIOx_RS -> BSRR = _GPIO_Pin_RS << 16; //   RS
	writeByte_LCD(0x1c);
	delayMks(50);
}

//  
void leftToRight_LCD() {
	_GPIOx_RS -> BSRR = _GPIO_Pin_RS << 16; //   RS
	_displaymode |= 2;
	writeByte_LCD(_displaymode);
	delayMks(50);
}

//  
void rightToLeft_LCD() {
	_GPIOx_RS -> BSRR = _GPIO_Pin_RS << 16; //   RS
	_displaymode &= 0b11111101;
	writeByte_LCD(_displaymode);
	delayMks(50);
}

//  
void autoscroll_LCD() {
	_GPIOx_RS -> BSRR = _GPIO_Pin_RS << 16; //   RS
	_displaymode |= 1;
	writeByte_LCD(_displaymode);
	delayMks(50);
}

//  
void noAutoscroll_LCD() {
	_GPIOx_RS -> BSRR = _GPIO_Pin_RS << 16; //   RS
	_displaymode &= 0b11111110;
	writeByte_LCD(_displaymode);
	delayMks(50);
}

//   
void createChar_LCD(uint8_t location, uint8_t charmap[]) {
	_GPIOx_RS -> BSRR = _GPIO_Pin_RS << 16; //   RS
	location &= 0x7;
	writeByte_LCD(0x40 | (location << 3));
	delayMks(50);
	_GPIOx_RS -> BSRR = _GPIO_Pin_RS; //   RS
	for (int i=0; i<8; i++) {
		writeByte_LCD(charmap[i]);
		delayMks(50);
	}
}



//    
void setCursor_LCD(uint8_t col, uint8_t row) {
	_GPIOx_RS -> BSRR = _GPIO_Pin_RS << 16; //   RS

	if( _row == 1 ) {
		//  
		if( col <= 8 ) writeByte_LCD(0x80 | (col - 1));
		else writeByte_LCD(0x80 | (col - 1 + 0x40));
	}
	else if( _row == 2 ) {
		//  
		if( row == 1 ) writeByte_LCD(0x80 | (col - 1));
		else writeByte_LCD(0x80 | (col - 1 + 0x40));
	}
	if( _row == 4 ) {
		if( row == 1 ) writeByte_LCD(0x80 | (col - 1));
		else if( row == 2 ) writeByte_LCD(0x80 | (col - 1 + 0x40));
		else if( row == 3 ) writeByte_LCD(0x80 | (col - 1 + _col));
		else if( row == 4 ) writeByte_LCD(0x80 | (col - 1 + 0x40 + _col));
	}
	delayMks(50);
}

//   
void home_LCD(void) {
	_GPIOx_RS -> BSRR = _GPIO_Pin_RS << 16; //   RS
	writeByte_LCD(0x2);
	delayMks(2000);
}

//  
void clear_LCD(void) {
	_GPIOx_RS -> BSRR = _GPIO_Pin_RS << 16; //   RS
	writeByte_LCD(0x1);
	delayMks(2000);
}

//   float
void print_float_LCD(float number, uint8_t digits) {

	//  
	if (number < 0.0) {
		write_LCD('-');
		number = -number;
	}

	// 
	float rounding = 0.5;
	for (uint8_t i=0; i<digits; ++i) rounding /= 10.0;
	number += rounding;

	//      
	uint32_t int_part = (uint32_t)number;
	float remainder = number - (float)int_part;
	print_uint_LCD(int_part, 10);

	//   
	  if (digits > 0) write_LCD('.');

	  while (digits-- > 0)
	  {
	    remainder *= 10.0;
	    uint16_t toPrint = (uint16_t)(remainder);
	    print_uint_LCD(toPrint, 10);
	    remainder -= toPrint;
	  }
}


//  
void print_str_LCD(const uint8_t str[]) {

	for( uint8_t i=0; i<200; i++ ){
		if( str[i] == 0 ) break;
		write_LCD(str[i]);
	}
}

//    uint32_t
void print_uint_LCD(uint32_t n, uint8_t base) {

	uint8_t buf[33]= {'\0'};
	uint8_t * str= buf +31;
	uint8_t c;
	do {
		c = n % base;
	    n /= base;
	    *--str = c < 10 ? c + '0' : c + 'A' - 10;
	  } while(n);

	for( uint8_t i=0; i<32; i++ ){
		if( str[i] == 0 ) break;
		write_LCD(str[i]);
	}
}

//    int32_t
void print_int_LCD(int32_t n, uint8_t base) {

	if ((base == 10) && (n < 0) ) {
		write_LCD('-');
		n = -n;
	}
	print_uint_LCD(n, base);
}

//     
void writeBuf_LCD(uint8_t * buf, uint16_t length) {
	_GPIOx_RS -> BSRR = _GPIO_Pin_RS; //   RS
	for( uint16_t i=0; i<length; i++ ) {
		if ( ((*(buf + i)) >= 0x80) && (_cirill != 0) ) writeByte_LCD( cirillAsciiTable[(*(buf + i)) - 0x80] );
		else writeByte_LCD((*(buf + i)));
		delayMks(50);
	}
}

//    
void write_LCD(uint8_t valChar) {
	_GPIOx_RS -> BSRR = _GPIO_Pin_RS; //   RS
	if ( (valChar>=0x80) && (_cirill != 0) ) writeByte_LCD( cirillAsciiTable[valChar - 0x80] );
	else writeByte_LCD(valChar);
	delayMks(50);
}

//  ,  rs, en, d4, d5, d6, d7 (,  ),    ,  ,   
void init_LCD( GPIO_TypeDef *GPIOx_RS, uint16_t GPIO_Pin_RS, GPIO_TypeDef *GPIOx_EN, uint16_t GPIO_Pin_EN,
		GPIO_TypeDef *GPIOx_D4, uint16_t GPIO_Pin_D4, GPIO_TypeDef *GPIOx_D5, uint16_t GPIO_Pin_D5,
		GPIO_TypeDef *GPIOx_D6, uint16_t GPIO_Pin_D6, GPIO_TypeDef *GPIOx_D7, uint16_t GPIO_Pin_D7,
		uint8_t col, uint8_t row, uint8_t cirill) {

	_GPIOx_RS = GPIOx_RS;
	_GPIO_Pin_RS = GPIO_Pin_RS;
	_GPIOx_EN = GPIOx_EN;
	_GPIO_Pin_EN = GPIO_Pin_EN;
	_GPIOx_D4 = GPIOx_D4;
	_GPIO_Pin_D4 = GPIO_Pin_D4;
	_GPIOx_D5 = GPIOx_D5;
	_GPIO_Pin_D5 = GPIO_Pin_D5;
	_GPIOx_D6 = GPIOx_D6;
	_GPIO_Pin_D6 = GPIO_Pin_D6;
	_GPIOx_D7 = GPIOx_D7;
	_GPIO_Pin_D7 = GPIO_Pin_D7;
	_col = col;
	_row = row;
	_cirill = cirill;

	//    
	uint32_t i;
	uint16_t d;

	//  EN
	i=0;
	d = _GPIO_Pin_EN;
	while(i < 16) {
		d = d >> 1;
		if(d == 0) break;
		i++;
	}
	if(i < 8) {
		_GPIOx_EN->CRL |= 0b0011 << (i * 4);
		_GPIOx_EN->CRL &= ~ (0b1100 << (i * 4));
	}
	else {
		_GPIOx_EN->CRH |= 0b0011 << ((i - 8) * 4);
		_GPIOx_EN->CRH &= ~ (0b1100 << ((i - 8) * 4));
	}
	_GPIOx_EN->ODR &= ~ (1 << i);

	//  RS
	i=0;
	d = _GPIO_Pin_RS;
	while(i < 16) {
		d = d >> 1;
		if(d == 0) break;
		i++;
	}
	if(i < 8) {
		_GPIOx_RS->CRL |= 0b0011 << (i * 4);
		_GPIOx_RS->CRL &= ~ (0b1100 << (i * 4));
	}
	else {
		_GPIOx_RS->CRH |= 0b0011 << ((i - 8) * 4);
		_GPIOx_RS->CRH &= ~ (0b1100 << ((i - 8) * 4));
	}
	_GPIOx_RS->ODR &= ~ (1 << i);

	//  D4
	i=0;
	d = _GPIO_Pin_D4;
	while(i < 16) {
		d = d >> 1;
		if(d == 0) break;
		i++;
	}
	if(i < 8) {
		_GPIOx_D4->CRL |= 0b0011 << (i * 4);
		_GPIOx_D4->CRL &= ~ (0b1100 << (i * 4));
	}
	else {
		_GPIOx_D4->CRH |= 0b0011 << ((i - 8) * 4);
		_GPIOx_D4->CRH &= ~ (0b1100 << ((i - 8) * 4));
	}
	_GPIOx_D4->ODR &= ~ (1 << i);

	//  D5
	i=0;
	d = _GPIO_Pin_D5;
	while(i < 16) {
		d = d >> 1;
		if(d == 0) break;
		i++;
	}
	if(i < 8) {
		_GPIOx_D5->CRL |= 0b0011 << (i * 4);
		_GPIOx_D5->CRL &= ~ (0b1100 << (i * 4));
	}
	else {
		_GPIOx_D5->CRH |= 0b0011 << ((i - 8) * 4);
		_GPIOx_D5->CRH &= ~ (0b1100 << ((i - 8) * 4));
	}
	_GPIOx_D5->ODR &= ~ (1 << i);

	//  D6
	i=0;
	d = _GPIO_Pin_D6;
	while(i < 16) {
		d = d >> 1;
		if(d == 0) break;
		i++;
	}
	if(i < 8) {
		_GPIOx_D6->CRL |= 0b0011 << (i * 4);
		_GPIOx_D6->CRL &= ~ (0b1100 << (i * 4));
	}
	else {
		_GPIOx_D6->CRH |= 0b0011 << ((i - 8) * 4);
		_GPIOx_D6->CRH &= ~ (0b1100 << ((i - 8) * 4));
	}
	_GPIOx_D6->ODR &= ~ (1 << i);

	//  D7
	i=0;
	d = _GPIO_Pin_D7;
	while(i < 16) {
		d = d >> 1;
		if(d == 0) break;
		i++;
	}
	if(i < 8) {
		_GPIOx_D7->CRL |= 0b0011 << (i * 4);
		_GPIOx_D7->CRL &= ~ (0b1100 << (i * 4));
	}
	else {
		_GPIOx_D7->CRH |= 0b0011 << ((i - 8) * 4);
		_GPIOx_D7->CRH &= ~ (0b1100 << ((i - 8) * 4));
	}
	_GPIOx_D7->ODR &= ~ (1 << i);


	//  
	delayMks(15000); //  15 

	_displaycontrol= 0xc;
	_displaymode= 6;

	_GPIOx_RS -> BSRR = _GPIO_Pin_RS << 16; //   RS

	//  3   
	_GPIOx_D7 -> BSRR = _GPIO_Pin_D7 << 16;
	_GPIOx_D6 -> BSRR = _GPIO_Pin_D6 << 16;
	_GPIOx_D5 -> BSRR = _GPIO_Pin_D5;
	_GPIOx_D4 -> BSRR = _GPIO_Pin_D4;

	delayNs(200); //  200 

	// 
	_GPIOx_EN -> BSRR = _GPIO_Pin_EN; //   EN
	delayNs(600); //  600 
	_GPIOx_EN -> BSRR = _GPIO_Pin_EN << 16; //   EN

	delayMks(4500); //  4,5 

	// 
	_GPIOx_EN -> BSRR = _GPIO_Pin_EN; //   EN
	delayNs(600); //  600 
	_GPIOx_EN -> BSRR = _GPIO_Pin_EN << 16; //   EN

	delayMks(4500); //  4,5 

	// 
	_GPIOx_EN -> BSRR = _GPIO_Pin_EN; //   EN
	delayNs(600); //  600 
	_GPIOx_EN -> BSRR = _GPIO_Pin_EN << 16; //   EN

	delayMks(4500); //  4,5 

	//  
	if( row < 2 ) writeByte_LCD(0x20); //  
	else writeByte_LCD(0x28); //  
	delayMks(50);

	//   
	writeByte_LCD(0x6);
	delayMks(50);

	//  ,  
	writeByte_LCD(_displaycontrol);
	delayMks(50);

	//  
	writeByte_LCD(0x1);
	delayMks(2000);

}

//    LCD
void writeByte_LCD(uint8_t dt) {

	//    
	if( (dt & 0x80) == 0 ) _GPIOx_D7 -> BSRR = _GPIO_Pin_D7 << 16;
	else _GPIOx_D7 -> BSRR = _GPIO_Pin_D7;
	if( (dt & 0x40) == 0 ) _GPIOx_D6 -> BSRR = _GPIO_Pin_D6 << 16;
	else _GPIOx_D6 -> BSRR = _GPIO_Pin_D6;
	if( (dt & 0x20) == 0 ) _GPIOx_D5 -> BSRR = _GPIO_Pin_D5 << 16;
	else _GPIOx_D5 -> BSRR = _GPIO_Pin_D5;
	if( (dt & 0x10) == 0 ) _GPIOx_D4 -> BSRR = _GPIO_Pin_D4 << 16;
	else _GPIOx_D4 -> BSRR = _GPIO_Pin_D4;

	delayNs(200); //  200 

	_GPIOx_EN -> BSRR = _GPIO_Pin_EN; //   EN

	delayNs(600); //  600 

	_GPIOx_EN -> BSRR = _GPIO_Pin_EN << 16; //   EN

	delayNs(20); //  20 

	//    
	if( (dt & 0x8) == 0 ) _GPIOx_D7 -> BSRR = _GPIO_Pin_D7 << 16;
	else _GPIOx_D7 -> BSRR = _GPIO_Pin_D7;
	if( (dt & 0x4) == 0 ) _GPIOx_D6 -> BSRR = _GPIO_Pin_D6 << 16;
	else _GPIOx_D6 -> BSRR = _GPIO_Pin_D6;
	if( (dt & 0x2) == 0 ) _GPIOx_D5 -> BSRR = _GPIO_Pin_D5 << 16;
	else _GPIOx_D5 -> BSRR = _GPIO_Pin_D5;
	if( (dt & 0x1) == 0 ) _GPIOx_D4 -> BSRR = _GPIO_Pin_D4 << 16;
	else _GPIOx_D4 -> BSRR = _GPIO_Pin_D4;

	delayNs(200); //  200 

	_GPIOx_EN -> BSRR = _GPIO_Pin_EN; //   EN

	delayNs(600); //  600 

	_GPIOx_EN -> BSRR = _GPIO_Pin_EN << 16; //   EN
}
